
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace HealthBarProject
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        internal GraphicsDeviceManager graphics;
        internal ContentManager content;
        SpriteBatch sb; // used to draw the health bar
        Model tiny; // the MS tiny modal
        Camera camera; // a camera for viewing the scene
        IUserInputService input; // handles user input
        Texture2D health; // holds the health texture


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // setup grid rendering component
            GridRenderer gr = new GridRenderer(this, 500, 500);
            gr.Offset = new Vector3(250, 0, 250) + (Vector3.Up * 250);
            gr.CellWidth = 25;
            gr.CellHeight = 25;
            gr.Pitch = MathHelper.PiOver2;
            this.Components.Add(gr);

            // setup the camera
            camera = new Camera(this);
            gr.Camera = this.camera;

            // create the spritebatch renderer
            sb = new SpriteBatch(this.graphics.GraphicsDevice);

            // create the user input object and ass it as a service
            input = new UserInputService(this);
            this.Services.AddService(typeof(IUserInputService), input);

            base.Initialize();
        }


        private void HandleKeyboardInput(GameTime gameTime)
        {

            if (input.KeyClick(Keys.F12)) this.graphics.ToggleFullScreen();
            if (input.KeyClick(Keys.Escape)) this.Exit();

            // get rotation
            float lt = input.IsKeyDown(Keys.Q) ? 1 : 0;
            float rt = input.IsKeyDown(Keys.E) ? 1 : 0;
            float rotation = (-lt + rt) * gameTime.ElapsedGameTime.Milliseconds;

            // get panning
            float val = 1f * gameTime.ElapsedGameTime.Milliseconds;
            Vector2 Horiz = Vector2.Zero;
            if (input.IsKeyDown(Keys.A)) Horiz.X = -val;
            if (input.IsKeyDown(Keys.D)) Horiz.X = val;

            Vector2 Vert = Vector2.Zero;

            if (input.IsKeyUp(Keys.LeftShift))
            {
                if (input.IsKeyDown(Keys.W)) Vert.Y = -val;
                if (input.IsKeyDown(Keys.S)) Vert.Y = val;
            }
            // calc diraction to move
            Vector2 pan = Horiz + Vert;
            if (pan != Vector2.Zero)
            {
                float rot = Helpers.GetRotationInRadians(0, 0, pan.X, pan.Y);
                pan.X = (float)Math.Cos(rot + -MathHelper.PiOver2);
                pan.Y = (float)Math.Sin(rot + -MathHelper.PiOver2);
                pan.Normalize();
                pan = pan * val;
            }

            // get zoom
            val = 0;
            if (input.IsKeyDown(Keys.LeftShift))
            {
                if (input.IsKeyDown(Keys.W)) val = 0.75f;
                if (input.IsKeyDown(Keys.S)) val = -0.75f;
            }
            val *= gameTime.ElapsedGameTime.Milliseconds;

            UpdateCamera(gameTime, pan, rotation, val);
        }

        private void HandleGamePadInput(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (input.BackClick(PlayerIndex.One)) this.Exit();

            Vector2 v2 = input.LeftThumbStick(PlayerIndex.One) * (0.75f * gameTime.ElapsedGameTime.Milliseconds);
            float lt = input.LeftTrigger(PlayerIndex.One);
            float rt = input.RightTrigger(PlayerIndex.One);
            float rotation = (lt + -rt) * gameTime.ElapsedGameTime.Milliseconds;

            // get zoom
            float val = 0;
            if (input.Up(PlayerIndex.One) == ButtonState.Pressed) val = -0.75f;
            if (input.Down(PlayerIndex.One) == ButtonState.Pressed) val = 0.75f;
            val *= gameTime.ElapsedGameTime.Milliseconds;

            UpdateCamera(gameTime, v2, rotation, val);
        }

        private void UpdateCamera(GameTime gameTime, Vector2 v2, float rotation, float zoom)
        {
            Vector3 pos = this.camera.Position;
            Vector3 dir = this.camera.Direction;

            v2.Y = -v2.Y;


            float direction = Helpers.GetRotationInRadians(pos.X, pos.Z, dir.X, dir.Z);
            float offsetdirection = Helpers.GetRotationInRadians(0, 0, v2.X, v2.Y);

            Vector3 newpos = Vector3.Zero;

            // ========= perform zoom ===============
            float val;
            float dist = Vector2.Distance(new Vector2(dir.X, dir.Z), new Vector2(pos.X, pos.Z));
            dist += zoom;

            // cap distances
            const float MinCameraDistance = 50;
            const float MaxCameraDistance = 1000;
            dist = dist < MinCameraDistance ? MinCameraDistance : dist;
            dist = dist > MaxCameraDistance ? MaxCameraDistance : dist;

            newpos.X = dir.X + ((float)Math.Cos(-direction) * dist);
            newpos.Z = dir.Z + ((float)Math.Sin(-direction) * dist);
            newpos.Y = dir.Y + dist;// (dist * 0.75f);
            pos = newpos;

            // ========== perform panning ==========
            newpos.X = (float)Math.Cos(direction + offsetdirection + MathHelper.PiOver2);
            newpos.Z = (float)Math.Sin(direction + offsetdirection + MathHelper.PiOver2);
            newpos.Y = 0;

            dist = Vector2.Distance(Vector2.Zero, v2);

            newpos *= (dist * (pos.Y * 0.0025f));
            dir += newpos;
            pos += newpos;

            // ========== perform rotation =======
            val = (rotation * ((MathHelper.Pi / 180f) * 0.25f));

            dist = Vector2.Distance(new Vector2(dir.X, dir.Z), new Vector2(pos.X, pos.Z));
            pos.X = dir.X + ((float)Math.Cos(-direction + -val) * dist);
            pos.Z = dir.Z + ((float)Math.Sin(-direction + -val) * dist);

            this.camera.Direction = dir;
            this.camera.Position = pos;
        }

        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                this.tiny = content.Load<Model>("tiny");
                this.health = content.Load<Texture2D>("health");
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // update inputs
            input.Update(gameTime);
            HandleGamePadInput(gameTime);
            HandleKeyboardInput(gameTime);
            camera.Update(gameTime);

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // draw components (IE the grid)
            base.Draw(gameTime);

            // draw the tiny mesh
            Helpers.DrawModel(gameTime, camera, tiny, Vector3.Zero, new Vector3(90, 0, 0));


            Vector3 pos;
            Viewport vp = this.graphics.GraphicsDevice.Viewport;
            Matrix world = Matrix.Identity;

            // position of health in 3D world space
            Vector3 worldpos = Vector3.Up * 275;

            // only draw health if the camera position is higher then the health bar position
            // uncommenting this check will cause undesired behavior
            if (camera.position.Y > worldpos.Y)
            {

                // project a point from the 3D world space to 2D screen space
                pos = vp.Project(worldpos, camera.ProjectionMatrix, camera.ViewMatrix, world);

                // calc the distance from the camera position to the desired 3D world space position
                // of the health bar
                float dist2cam = Vector3.Distance(worldpos, camera.position);

                // calc the scale of the health bar and limit it's value between 0.5 and 1
                float scale = Math.Max(0.5f, dist2cam * 0.005f);
                scale = Math.Min(1, scale);

                // draw the health bar for the tiny modal
                sb.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
                sb.Draw(this.health,
                        new Vector2(pos.X, pos.Y),
                        null,
                        Color.White,
                    //new Color(255, 255, 255, (byte)(scale * 255)), // uncomment to fade based on scale
                        0f,
                        new Vector2(64, 16),
                        scale,
                        SpriteEffects.None,
                        0f);
                sb.End();
            }

        }
    }
}