using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.Reflection;


namespace HealthBarProject
{
    public static class Helpers
    {
      
        private const float OneRad = MathHelper.Pi / 180f;

        public static void DrawModel(GameTime gameTime, Camera cam, Model m)
        {
            DrawModel(gameTime, cam, m, Vector3.Zero, Vector3.Zero, Color.White, 1);
        }

        public static void DrawModel(GameTime gameTime, Camera cam, Model m, Vector3 pos)
        {
            DrawModel(gameTime, cam, m, pos, Vector3.Zero, Color.White, 1);
        }

        public static void DrawModel(GameTime gameTime, Camera cam, Model m, Vector3 pos, Vector3 rotation)
        {
            DrawModel(gameTime, cam, m, pos, rotation, Color.White, 1);
        }

        public static void DrawModel(GameTime gameTime, Camera camera, Model m, Vector3 pos, Vector3 rotation, Color color)
        {
            DrawModel(gameTime, camera, m, pos, rotation, color, 1);             
        }

        public static void DrawModel(GameTime gameTime, Camera cam, Model m, Vector3 pos, Vector3 rotation, Color col, float alpha)
        {
            Matrix[] transforms = new Matrix[m.Bones.Count];
            m.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in m.Meshes)
            {
                foreach (Effect effect in mesh.Effects)
                {
                   // Matrix World;
                    switch (effect.GetType().Name)
                    {
                        case "BasicEffect":
                            BasicEffect fx = (BasicEffect)effect;
                            fx.EnableDefaultLighting();
                            fx.Alpha = alpha;
                            fx.DiffuseColor = col.ToVector3();
                            fx.View = cam.ViewMatrix;
                            fx.Projection = cam.ProjectionMatrix;
                            fx.World = transforms[mesh.ParentBone.Index] *
                                Matrix.CreateRotationX(rotation.X * OneRad) *
                                Matrix.CreateRotationY(rotation.Y * OneRad) *
                                Matrix.CreateRotationZ(rotation.Z * OneRad) *
                                    Matrix.CreateTranslation(pos);
                            break;

                                }
                }

                mesh.Draw();
            }
        }

        public static float GetRotationInDegrees(float X1, float Y1, float X2, float Y2)
        {
            float Rot;
            Rot = (float)Math.Atan2(Y2 - Y1, X2 - X1);
            if (Rot < 0) Rot += 2.0F * (float)Math.PI;
            Rot = (float)(Rot * 180.0 / (float)Math.PI);
            return Rot;
        }

        public static float GetRotationInRadians(float X1, float Y1, float X2, float Y2)
        {
            float rot;
            rot = (float)Math.Atan2(Y2 - Y1, X2 - X1);
            if (rot < 0) rot += 2.0F * (float)Math.PI;
            return rot;
        }

        public static Rectangle CenterRectangle(Rectangle r, int x, int y)
        {
            return new Rectangle(x - (r.Width / 2), y - (r.Height / 2), r.Width, r.Height);
        }

        /// <summary>
        /// Finds a component of a specified type.
        /// </summary>
        /// <typeparam name="T">The type of component to search for.</typeparam>
        /// <param name="Items">A collection of components to be searched.</param>
        /// <returns>If found returns the component reference else returns null.</returns>
        public static T FindComponent<T>(GameComponentCollection Items) where T : IGameComponent
        {
            for (int idx = 0; idx <= Items.Count - 1; idx++)
            {
                if (Items[idx].GetType() == typeof(T))
                {
                    return (T)Items[idx];
                }
            }

            return default(T);
        }
      
        public static bool IsNumeric(string value)
        {
            if (value == null) return false;
            value = value.Trim();                                 
            if (value == string.Empty) return false;
            if (char.IsSymbol(value[0]) == false && char.IsNumber(value[0]) == false) return false;
            if (char.IsSymbol(value[0]) && value.Length > 1 && char.IsNumber(value[1]) == false) return false;
            return true;
        }

        public static System.Collections.Specialized.StringCollection CloneStringCollection(System.Collections.Specialized.StringCollection items)
        {
            System.Collections.Specialized.StringCollection list = new System.Collections.Specialized.StringCollection();
            for (int idx = 0; idx < items.Count - 1; idx++)
            {
                list.Add(items[idx]);
            }
            return list;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="quat"></param>
        /// <param name="vector"></param>
        /// <returns></returns>
        public static Vector3 Vec3QuatMultiply(Quaternion quat, Vector3 vector)
        {
            // nVidia SDK implementation
            Vector3 uv, uuv;
            Vector3 qvec = new Vector3(quat.X, quat.Y, quat.Z);
            Vector3.Cross(ref qvec, ref vector, out uv);
            // uv = qvec.Cross(vector);
            Vector3.Cross(ref qvec, ref uv, out uuv);
            // uuv = qvec.Cross(uv);
            uv *= (2.0f * quat.W);
            uuv *= 2.0f;

            return vector + uv + uuv;

            // get the rotation matrix of the Quaternion and multiply it times the vector
            //return quat.ToRotationMatrix() * vector;
        }  
    }
}
