#define Win32

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace HealthBarProject
{
    public class UserInputService : GameComponent, IUserInputService
    {
        #region Fields
        private ButtonState[] _A = new ButtonState[4];
        private ButtonState[] _B = new ButtonState[4];
        private ButtonState[] _X = new ButtonState[4];
        private ButtonState[] _Y = new ButtonState[4];
        private ButtonState[] _Back = new ButtonState[4];
        private ButtonState[] _Start = new ButtonState[4];
        private ButtonState[] _LeftSholder = new ButtonState[4];
        private ButtonState[] _LeftStick = new ButtonState[4];
        private ButtonState[] _RightSholder = new ButtonState[4];
        private ButtonState[] _RightStick = new ButtonState[4];
        private ButtonState[] _Left = new ButtonState[4];
        private ButtonState[] _Right = new ButtonState[4];
        private ButtonState[] _Up = new ButtonState[4];
        private ButtonState[] _Down = new ButtonState[4];
        private Vector2[] _LeftThumbStick = new Vector2[4];
        private Vector2[] _RightThumbStick = new Vector2[4];
        private float[] _LeftTrigger = new float[4];
        private float[] _RightTrigger = new float[4];
        private bool[] _IsConnected = new bool[4];
        private KeyboardState _kbState = new KeyboardState();
        private MouseState _mouseState = new MouseState();

        private ButtonState[] _PreviousA = new ButtonState[4];
        private ButtonState[] _PreviousB = new ButtonState[4];
        private ButtonState[] _PreviousX = new ButtonState[4];
        private ButtonState[] _PreviousY = new ButtonState[4];
        private ButtonState[] _PreviousBack = new ButtonState[4];
        private ButtonState[] _PreviousStart = new ButtonState[4];
        private ButtonState[] _PreviousLeftSholder = new ButtonState[4];
        private ButtonState[] _PreviousLeftStick = new ButtonState[4];
        private ButtonState[] _PreviousRightSholder = new ButtonState[4];
        private ButtonState[] _PreviousRightStick = new ButtonState[4];
        private ButtonState[] _PreviousLeft = new ButtonState[4];
        private ButtonState[] _PreviousRight = new ButtonState[4];
        private ButtonState[] _PreviousUp = new ButtonState[4];
        private ButtonState[] _PreviousDown = new ButtonState[4];
        private Vector2[] _PreviousLeftThumbStick = new Vector2[4];
        private Vector2[] _PreviousRightThumbStick = new Vector2[4];
        private float[] _PreviousLeftTrigger = new float[4];
        private float[] _PreviousRightTrigger = new float[4];
        private bool[] _PreviousIsConnected = new bool[4];
        private KeyboardState _prevkbState = new KeyboardState();
        private MouseState _prevmouseState = new MouseState();

        #endregion


        public UserInputService(Game game)
            : base(game)
        {
        }

        public override void Update(GameTime gameTime)
        {
            for (int idx = 0; idx <= 3; idx++)
            {
                GamePadState state;
                state = GamePad.GetState((PlayerIndex)idx, GamePadDeadZone.Circular);

                UpdateState(idx, state);
            }

            _prevkbState = _kbState;
            _kbState = Keyboard.GetState();

            _prevmouseState = _mouseState;
            _mouseState = Mouse.GetState();

            base.Update(gameTime);
        }

        private void UpdateState(int idx, GamePadState state)
        {
            _PreviousA[idx] = _A[idx];
            _PreviousB[idx] = _B[idx];
            _PreviousX[idx] = _X[idx];
            _PreviousY[idx] = _Y[idx];
            _PreviousBack[idx] = _Back[idx];
            _PreviousStart[idx] = _Start[idx];
            _PreviousLeftSholder[idx] = _LeftSholder[idx];
            _PreviousLeftStick[idx] = _LeftStick[idx];
            _PreviousRightSholder[idx] = _RightSholder[idx];
            _PreviousRightStick[idx] = _RightStick[idx];
            _PreviousLeft[idx] = _Left[idx];
            _PreviousRight[idx] = _Right[idx];
            _PreviousUp[idx] = _Up[idx];
            _PreviousDown[idx] = _Down[idx];
            _PreviousLeftThumbStick[idx] = _LeftThumbStick[idx];
            _PreviousRightThumbStick[idx] = _RightThumbStick[idx];
            _PreviousLeftTrigger[idx] = _LeftTrigger[idx];
            _PreviousRightTrigger[idx] = _RightTrigger[idx];
            _PreviousIsConnected[idx] = _IsConnected[idx];

            _A[idx] = state.Buttons.A;
            _B[idx] = state.Buttons.B;
            _X[idx] = state.Buttons.X;
            _Y[idx] = state.Buttons.Y;
            _Back[idx] = state.Buttons.Back;
            _Start[idx] = state.Buttons.Start;
            _LeftSholder[idx] = state.Buttons.LeftShoulder;
            _LeftStick[idx] = state.Buttons.LeftStick;
            _RightSholder[idx] = state.Buttons.RightShoulder;
            _RightStick[idx] = state.Buttons.RightStick;
            _Left[idx] = state.DPad.Left;
            _Right[idx] = state.DPad.Right;
            _Up[idx] = state.DPad.Up;
            _Down[idx] = state.DPad.Down;
            _LeftThumbStick[idx] = state.ThumbSticks.Left;
            _RightThumbStick[idx] = state.ThumbSticks.Right;
            _LeftTrigger[idx] = state.Triggers.Left;
            _RightTrigger[idx] = state.Triggers.Right;
            _IsConnected[idx] = state.IsConnected;
        }


        #region IUserInputService Members

        public ButtonState A(PlayerIndex index)
        {
            return _A[(int)index];
        }

        public ButtonState B(PlayerIndex index)
        {
            return _B[(int)index];
        }

        public ButtonState X(PlayerIndex index)
        {
            return _X[(int)index];
        }

        public ButtonState Y(PlayerIndex index)
        {
            return _Y[(int)index];
        }

        public ButtonState Back(PlayerIndex index)
        {
            return _Back[(int)index];
        }

        public ButtonState Start(PlayerIndex index)
        {
            return _Start[(int)index];
        }

        public ButtonState LeftSholder(PlayerIndex index)
        {
            return _LeftSholder[(int)index];
        }

        public ButtonState LeftStick(PlayerIndex index)
        {
            return _LeftStick[(int)index];
        }

        public ButtonState RightSholder(PlayerIndex index)
        {
            return _RightSholder[(int)index];
        }

        public ButtonState RightStick(PlayerIndex index)
        {
            return _RightStick[(int)index];
        }

        public ButtonState Left(PlayerIndex index)
        {
            return _Left[(int)index];
        }

        public ButtonState Right(PlayerIndex index)
        {
            return _Right[(int)index];
        }

        public ButtonState Up(PlayerIndex index)
        {
            return _Up[(int)index];
        }

        public ButtonState Down(PlayerIndex index)
        {
            return _Down[(int)index];
        }

        public Vector2 LeftThumbStick(PlayerIndex index)
        {
            return _LeftThumbStick[(int)index];
        }

        public Vector2 RightThumbStick(PlayerIndex index)
        {
            return _RightThumbStick[(int)index];
        }

        public float LeftTrigger(PlayerIndex index)
        {
            return _LeftTrigger[(int)index];
        }

        public float RightTrigger(PlayerIndex index)
        {
            return _RightTrigger[(int)index];
        }

        public bool IsConnected(PlayerIndex index)
        {
            return _IsConnected[(int)index];
        }

        public void A(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousA[i] = _A[i];
            _A[i] = state;
        }

        public void B(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousB[i] = _B[i];
            _B[i] = state;
        }

        public void X(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousX[i] = _X[i];
            _X[i] = state;
        }

        public void Y(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousY[i] = _Y[i];
            _Y[i] = state;
        }

        public void Back(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousBack[i] = _Back[i];
            _Back[i] = state;
        }

        public void Start(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousStart[i] = _Start[i];
            _Start[i] = state;
        }

        public void LeftSholder(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousLeftSholder[i] = _LeftSholder[i];
            _LeftSholder[i] = state;
        }

        public void LeftStick(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousLeftStick[i] = _LeftStick[i];
            _LeftStick[i] = state;
        }

        public void RightSholder(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousRightSholder[i] = _RightSholder[i];
            _RightSholder[i] = state;
        }

        public void RightStick(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousRightStick[i] = _RightStick[i];
            _RightStick[i] = state;
        }

        public void Left(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousLeft[i] = _Left[i];
            _Left[i] = state;
        }

        public void Right(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousRight[i] = _Right[i];
            _Right[i] = state;
        }

        public void Up(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousUp[i] = _Up[i];
            _Up[i] = state;
        }

        public void Down(PlayerIndex index, ButtonState state)
        {
            int i = (int)index;
            _PreviousDown[i] = _Down[i];
            _Down[i] = state;
        }

        public void LeftThumbStick(PlayerIndex index, Vector2 value)
        {
            int i = (int)index;
            _PreviousLeftThumbStick[i] = _LeftThumbStick[i];
            _LeftThumbStick[i] = value;
        }

        public void RightThumbStick(PlayerIndex index, Vector2 value)
        {
            int i = (int)index;
            _PreviousRightThumbStick[i] = _RightThumbStick[i];
            _RightThumbStick[i] = value;
        }

        public void LeftTrigger(PlayerIndex index, float value)
        {
            int i = (int)index;
            _PreviousLeftTrigger[i] = _LeftTrigger[i];
            _LeftTrigger[i] = value;
        }

        public void RightTrigger(PlayerIndex index, float value)
        {
            int i = (int)index;
            _PreviousRightTrigger[i] = _RightTrigger[i];
            _RightTrigger[i] = value;
        }

        public void IsConnected(PlayerIndex index, bool value)
        {
            int i = (int)index;
            _PreviousIsConnected[i] = _IsConnected[i];
            _IsConnected[i] = value;
        }

        public ButtonState PreviousA(PlayerIndex index)
        {
            return _PreviousA[(int)index];
        }

        public ButtonState PreviousB(PlayerIndex index)
        {
            return _PreviousB[(int)index];
        }

        public ButtonState PreviousX(PlayerIndex index)
        {
            return _PreviousX[(int)index];
        }

        public ButtonState PreviousY(PlayerIndex index)
        {
            return _PreviousY[(int)index];
        }

        public ButtonState PreviousBack(PlayerIndex index)
        {
            return _PreviousBack[(int)index];
        }

        public ButtonState PreviousStart(PlayerIndex index)
        {
            return _PreviousStart[(int)index];
        }

        public ButtonState PreviousLeftSholder(PlayerIndex index)
        {
            return _PreviousLeftSholder[(int)index];
        }

        public ButtonState PreviousLeftStick(PlayerIndex index)
        {
            return _PreviousLeftStick[(int)index];
        }

        public ButtonState PreviousRightSholder(PlayerIndex index)
        {
            return _PreviousRightSholder[(int)index];
        }

        public ButtonState PreviousRightStick(PlayerIndex index)
        {
            return _PreviousRightStick[(int)index];
        }

        public ButtonState PreviousLeft(PlayerIndex index)
        {
            return _PreviousLeft[(int)index];
        }

        public ButtonState PreviousRight(PlayerIndex index)
        {
            return _PreviousRight[(int)index];
        }

        public ButtonState PreviousUp(PlayerIndex index)
        {
            return _PreviousUp[(int)index];
        }

        public ButtonState PreviousDown(PlayerIndex index)
        {
            return _PreviousDown[(int)index];
        }

        public Vector2 PreviousLeftThumbStick(PlayerIndex index)
        {
            return _PreviousLeftThumbStick[(int)index];
        }

        public Vector2 PreviousRightThumbStick(PlayerIndex index)
        {
            return _PreviousRightThumbStick[(int)index];
        }

        public float PreviousLeftTrigger(PlayerIndex index)
        {
            return _PreviousLeftTrigger[(int)index];
        }

        public float PreviousRightTrigger(PlayerIndex index)
        {
            return _PreviousRightTrigger[(int)index];
        }

        public bool PreviousIsConnected(PlayerIndex index)
        {
            return _PreviousIsConnected[(int)index];
        }

        public bool IsKeyDown(Keys key)
        {
            return _kbState.IsKeyDown(key);
        }

        public bool IsKeyUp(Keys key)
        {
            return _kbState.IsKeyUp(key);
        }

        public bool WasKeyDown(Keys key)
        {
            return _prevkbState[key] == KeyState.Down;
        }

        public bool WasKeyUp(Keys key)
        {
            return _prevkbState[key] == KeyState.Up;
        }

        public Keys[] PressedKeys()
        {
            return _kbState.GetPressedKeys();
        }

        public Keys[] PreviouslyPressedKeys()
        {
            return _prevkbState.GetPressedKeys();
        }

        public bool KeyClick(Keys key)
        {
            return _prevkbState.IsKeyDown(key) && _kbState.IsKeyUp(key);
        }

#if Win32
        public bool IsLeftMouseButtonDown
        {
            get { return _mouseState.LeftButton == ButtonState.Pressed; }
        }

        public bool IsRightMouseButtonDown
        {
            get { return _mouseState.RightButton == ButtonState.Pressed; }
        }

        public bool IsMiddleMouseButtonDown
        {
            get { return _mouseState.MiddleButton == ButtonState.Pressed; }
        }

        public bool IsXButton1MouseButtonDown
        {
            get { return _mouseState.XButton1 == ButtonState.Pressed; }
        }

        public bool IsXButton2MouseButtonDown
        {
            get { return _mouseState.XButton2 == ButtonState.Pressed; }
        }

        public bool IsLeftMouseButtonUp
        {
            get { return _mouseState.LeftButton == ButtonState.Released; }
        }

        public bool IsRightMouseButtonUp
        {
            get { return _mouseState.RightButton == ButtonState.Released; }
        }

        public bool IsMiddleMouseButtonUp
        {
            get { return _mouseState.MiddleButton == ButtonState.Released; }
        }

        public bool IsXButton1MouseButtonUp
        {
            get { return _mouseState.XButton1 == ButtonState.Released; }
        }

        public bool IsXButton2MouseButtonUp
        {
            get { return _mouseState.XButton2 == ButtonState.Released; }
        }

        public int MouseX
        {
            get { return _mouseState.X; }
        }

        public int MouseY
        {
            get { return _mouseState.Y; }
        }

        public int MouseScrollWheel
        {
            get { return _mouseState.ScrollWheelValue; }
        }

        public bool WasLeftMouseButtonDown
        {
            get { return _prevmouseState.LeftButton == ButtonState.Pressed; }
        }

        public bool WasRightMouseButtonDown
        {
            get { return _prevmouseState.RightButton == ButtonState.Pressed; }
        }

        public bool WasMiddleMouseButtonDown
        {
            get { return _prevmouseState.MiddleButton == ButtonState.Pressed; }
        }

        public bool WasXButton1MouseButtonDown
        {
            get { return _prevmouseState.XButton1 == ButtonState.Pressed; }
        }

        public bool WasXButton2MouseButtonDown
        {
            get { return _prevmouseState.XButton2 == ButtonState.Pressed; }
        }

        public bool WasLeftMouseButtonUp
        {
            get { return _prevmouseState.LeftButton == ButtonState.Released; }
        }

        public bool WasRightMouseButtonUp
        {
            get { return _prevmouseState.RightButton == ButtonState.Released; }
        }

        public bool WasMiddleMouseButtonUp
        {
            get { return _prevmouseState.MiddleButton == ButtonState.Released; }
        }

        public bool WasXButton1MouseButtonUp
        {
            get { return _prevmouseState.XButton1 == ButtonState.Released; }
        }

        public bool WasXButton2MouseButtonUp
        {
            get { return _prevmouseState.XButton2 == ButtonState.Released; }
        }

        public int PreviousMouseX
        {
            get { return _prevmouseState.X; }
        }

        public int PreviousMouseY
        {
            get { return _prevmouseState.Y; }
        }

        public int PreviousMouseScrollWheel
        {
            get { return _prevmouseState.ScrollWheelValue; }
        }

        public bool LeftMouseButtonClicked
        {
            get
            {
                return _prevmouseState.LeftButton == ButtonState.Pressed &&
                       _mouseState.LeftButton == ButtonState.Released;
            }
        }

        public bool RightMouseButtonClicked
        {
            get
            {
                return _prevmouseState.RightButton == ButtonState.Pressed &&
                       _mouseState.RightButton == ButtonState.Released;
            }
        }

        public bool MiddleMouseButtonClicked
        {
            get
            {
                return _prevmouseState.MiddleButton == ButtonState.Pressed &&
                       _mouseState.MiddleButton == ButtonState.Released;
            }
        }

        public bool XButton1MouseButtonClicked
        {
            get
            {
                return _prevmouseState.XButton1 == ButtonState.Pressed &&
                       _mouseState.XButton1 == ButtonState.Released;
            }
        }

        public bool XButton2MouseButtonClicked
        {
            get
            {
                return _prevmouseState.XButton2 == ButtonState.Pressed &&
                       _mouseState.XButton2 == ButtonState.Released;
            }
        }

        public bool MouseXChanged
        {
            get { return _prevmouseState.X != _mouseState.X; }
        }

        public bool MouseYChanged
        {
            get { return _prevmouseState.Y != _mouseState.Y; }
        }

        public bool MouseScrollWheelChanged
        {
            get { return _prevmouseState.ScrollWheelValue != _mouseState.ScrollWheelValue; }
        }
#endif

        public bool AClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousA[idx] == ButtonState.Pressed && _A[idx] == ButtonState.Released;
        }

        public bool BClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousB[idx] == ButtonState.Pressed && _B[idx] == ButtonState.Released;
        }

        public bool XClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousX[idx] == ButtonState.Pressed && _X[idx] == ButtonState.Released;
        }

        public bool YClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousY[idx] == ButtonState.Pressed && _Y[idx] == ButtonState.Released;
        }

        public bool BackClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousBack[idx] == ButtonState.Pressed && _Back[idx] == ButtonState.Released;
        }

        public bool StartClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousStart[idx] == ButtonState.Pressed && _Start[idx] == ButtonState.Released;
        }

        public bool LeftSholderClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousLeftSholder[idx] == ButtonState.Pressed && _LeftSholder[idx] == ButtonState.Released;
        }

        public bool LeftStickClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousLeftStick[idx] == ButtonState.Pressed && _LeftStick[idx] == ButtonState.Released;
        }

        public bool RightSholderClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousRightSholder[idx] == ButtonState.Pressed && _RightSholder[idx] == ButtonState.Released;
        }

        public bool RightStickClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousRightStick[idx] == ButtonState.Pressed && _RightStick[idx] == ButtonState.Released;
        }

        public bool LeftClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousLeft[idx] == ButtonState.Pressed && _Left[idx] == ButtonState.Released;
        }

        public bool RightClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousRight[idx] == ButtonState.Pressed && _Right[idx] == ButtonState.Released;
        }

        public bool UpClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousUp[idx] == ButtonState.Pressed && _Up[idx] == ButtonState.Released;
        }

        public bool DownClick(PlayerIndex index)
        {
            int idx = (int)index;
            return _PreviousDown[idx] == ButtonState.Pressed && _Down[idx] == ButtonState.Released;
        }

        #endregion
    }
}
